;; The square root of 2 can be written as an infinite continued
;; fraction.

;; 2 = 1 + 
;; 1

;; 2 + 
;; 1

;; 2 + 
;; 1

;; 2 + 
;; 1

;; 2 + ...

;; The infinite continued fraction can be written, 2 = [1;(2)], (2)
;; indicates that 2 repeats ad infinitum. In a similar way, 23 =
;; [4;(1,3,1,8)].

;; It turns out that the sequence of partial values of continued
;; fractions for square roots provide the best rational
;; approximations. Let us consider the convergents for 2.

;; 1 + 
;; 1

;; = 3/2

;; 2
 
;; 1 + 
;; 1

;; = 7/5
;; 2 + 
;; 1


;; 2
 
;; 1 + 
;; 1

;; = 17/12
;; 2 + 
;; 1

 
;; 2 + 
;; 1

 

;; 2
 
;; 1 + 
;; 1

;; = 41/29
;; 2 + 
;; 1

;; 2 + 
;; 1

 
;; 2 + 
;; 1

 

;; 2
 

;; Hence the sequence of the first ten convergents for 2 are:
;; 1, 3/2, 7/5, 17/12, 41/29, 99/70, 239/169, 577/408, 1393/985, 3363/2378, ...

;; What is most surprising is that the important mathematical constant,
;; e = [2; 1,2,1, 1,4,1, 1,6,1 , ... , 1,2k,1, ...].

;; The first ten terms in the sequence of convergents for e are:
;; 2, 3, 8/3, 11/4, 19/7, 87/32, 106/39, 193/71, 1264/465, 1457/536, ...

;; The sum of digits in the numerator of the 10th convergent is
;; 1+4+5+7=17.

;; Find the sum of digits in the numerator of the 100th convergent of
;; the continued fraction for e


(defun continued-fraction (depth numerator rep-seq)
  (labels ((cont-fract (dep)
	     (cond ((= dep 0)
		    0)
		   (t
		    (/ 1 ;; numerator ;; 1
		       (+ 
			(funcall rep-seq dep)
			(cont-fract (- dep 1))))))))
    (+ numerator (cont-fract depth))))

;; sqrt(2) = [1;(2)]
(defun seq-sqrt-2 ()
  (lambda (dep)
    2))

;; (continued-fraction 0 1 (seq-sqrt-2))
;; (continued-fraction 1 1 (seq-sqrt-2))
;; (continued-fraction 2 1 (seq-sqrt-2))
;; (continued-fraction 3 1 (seq-sqrt-2))
;; (continued-fraction 4 1 (seq-sqrt-2))
;; ...
;; (continued-fraction 9 1 (seq-sqrt-2))

;; (* 1.0 (continued-fraction 100 1 (seq-sqrt-2)))
;; 1.4142135
;; (sqrt 2)
;; 1.4142135

(defun seq-sqrt-23 ()
  (let ((nums '(1 3 1 8))
	(pos 0))
    (lambda (depth)
      (prog1
	  (nth (mod pos (length nums)) nums)
;; 	(format t "returning: depth:~a pos:~a = ~a~&"
;; 		depth
;; 		pos
;; 		(nth (mod pos (length nums)) nums))
	(incf pos)))))

;;(* 1.0 (continued-fraction 100 4 (seq-sqrt-23)))
;; 4.7958317
;;(sqrt 23.0)
;; 4.7958317;


(defun seq-e ()
  (let ((pos 1)
	(nn 1))
    (lambda (depth)
      (declare (ignore depth))
      (prog1
	  ;; on pos=2, pos=5, pos=8
	  (if (= 0 (mod (- pos 2) 3))
	      (prog1
		  (* 2 nn)
		;;(format t "depth:~a pos:~a nn:~a => ~a~&"
		;; depth pos nn (* 2 nn))
		(incf nn))
	      1)
	(incf pos)))))

(continued-fraction 0 2 (seq-e))
(continued-fraction 1 2 (seq-e))
(continued-fraction 2 2 (seq-e))
(continued-fraction 3 2 (seq-e))
(continued-fraction 4 2 (seq-e))
(continued-fraction 5 2 (seq-e))
(continued-fraction 6 2 (seq-e))
(continued-fraction 7 2 (seq-e))

(defun explode-number (num)
  (loop for ch across (format nil "~d" num)
	collect (read-from-string (format nil "~a" ch))))

;; (explode-number 123)

(apply #'+ (explode-number (numerator (continued-fraction 99 2 (seq-e)))))

;; (6963524437876961749120273824619538346438023188214475670667/2561737478789858711161539537921323010415623148113041714756
